package ch.lantersoft.grd.protector;

import java.awt.Color;
import java.awt.LinearGradientPaint;
import java.awt.Paint;

import ch.lantersoft.grd.Fraction;
import ch.lantersoft.grd.Model;

public class LinearGradientProtector extends MultipleGradientProtector {

	private Model model;
	
	private float startX = 10;
	private float startY = 10;
	private float endX = 90;
	private float endY = 90;
	
	private boolean startXperc = true;
	private boolean startYperc = true;
	private boolean endXperc = true;
	private boolean endYperc = true;
	
	private boolean startAdjust = false; // never both true
	private boolean endAdjust = false;
	
	public LinearGradientProtector(Model model) {
		this.model = model;
	}
	
	@Override
	public String getName() {
		return "LinearGradientPaint";
	}

	@Override
	public Paint getPaint() {
		if (!isValid()) return null;
		int n = 0;
		Fraction[] fractions = getFractions();
		for (Fraction f:fractions) {
			if (f.isEnabled()) n++;
		}
		if (n<2) return null;
		float[] fracs = new float[n];
		Color[] colors = new Color[n];
		for (int i=0, j=0;i<fractions.length;i++) {
			if (fractions[i].isEnabled()) {
				fracs[j] = fractions[i].getFraction();
				colors[j] = fractions[i].getColor();
				j++;
			}
		}
		for (int i=0;i<n-1;i++) {
			if (fracs[i+1]>.5f) break;
			if (fracs[i]>=fracs[i+1]) {
				fracs[i+1] = fracs[i]+.001f;
			}
		}
		for (int i=n-1;i>0;i--) {
			if (fracs[i]<.5f) break;
			if (fracs[i-1]>=fracs[i]) {
				fracs[i-1] = fracs[i]-.001f;
			}
		}
		if (fracs[fracs.length-1]>1f) return null;
		float width = model.getWidth();
		float height = model.getHeight();
		float sx = startXperc?width*startX/100:startX;
		float sy = startYperc?height*startY/100:startY;
		float ex = endXperc?width*endX/100:endX;
		float ey = endYperc?height*endY/100:endY;
		if (sx==ex && sy==ey) return null;
		LinearGradientPaint paint = new LinearGradientPaint(
				sx,sy,ex,ey,
				fracs,colors,
				getCycleMode().getCycleMethod()
				);
		return paint;
	}
	
	@Override
	public String getCode() {
		int n = 0;
		Fraction[] fractions = getFractions();
		for (Fraction f:fractions) {
			if (f.isEnabled()) n++;
		}
		StringBuffer fracs = new StringBuffer();
		StringBuffer colors = new StringBuffer();
		
		int iMax = fractions.length - 1;
        for (int i = 0; ; i++) {
        	fracs.append(fractions[i].getFraction()+"f");
        	colors.append("new Color("+fractions[i].getRed()+", "+fractions[i].getGreen()+", "+fractions[i].getBlue()+")");
		    if (i == iMax) break;
		    fracs.append(", ");
		    colors.append(", ");
        }
		float width = model.getWidth();
		float height = model.getHeight();
		float sx = startXperc?width*startX/100:startX;
		float sy = startYperc?height*startY/100:startY;
		float ex = endXperc?width*endX/100:endX;
		float ey = endYperc?height*endY/100:endY;
		String ret = "new LinearGradientPaint(\n"
					+ "	" + sx+"f, "+sy+"f, "+ex+"f, "+ey+"f,\n"
					+ "	" + "new float[] {" + fracs.toString() + "},\n"
					+ "	" + "new Color[] {" + colors.toString() + "},\n"
					+ "	" + "CycleMethod."+getCycleMode().getCycleMethod().toString() + ");";
		return ret;
	}
	
	@Override
	public void click(int x, int y) {
		if (startAdjust) {
			startX = startXperc?x*100f/model.getWidth():x;
			startY = startYperc?y*100f/model.getHeight():y;
			startAdjust = false;
			fireStateChanged();
		} else if (endAdjust) {
			endX = endXperc?x*100f/model.getWidth():x;
			endY = endYperc?y*100f/model.getHeight():y;
			endAdjust = false;
			fireStateChanged();
		}
	}

	@Override
	public void move(int x, int y) {
		if (startAdjust) {
			startX = startXperc?x*100f/model.getWidth():x;
			startY = startYperc?y*100f/model.getHeight():y;
			fireStateChanged();
		} else if (endAdjust) {
			endX = endXperc?x*100f/model.getWidth():x;
			endY = endYperc?y*100f/model.getHeight():y;
			fireStateChanged();
		}
	}
	
	public float getStartX() {
		return startX;
	}

	public void setStartX(float startX, boolean perc) {
		this.startX = startX;
		this.startXperc = perc;
		fireStateChanged();
	}
	
	public boolean isStartXPerc() {
		return startXperc;
	}

	public float getStartY() {
		return startY;
	}

	public void setStartY(float startY, boolean perc) {
		this.startY = startY;
		this.startYperc = perc;
		fireStateChanged();
	}
	
	public boolean isStartYPerc() {
		return startYperc;
	}

	public float getEndX() {
		return endX;
	}

	public void setEndX(float endX, boolean perc) {
		this.endX = endX;
		this.endXperc = perc;
		fireStateChanged();
	}
	
	public boolean isEndXPerc() {
		return endXperc;
	}

	public float getEndY() {
		return endY;
	}

	public void setEndY(float endY, boolean perc) {
		this.endY = endY;
		this.endYperc = perc;
		fireStateChanged();
	}
	
	public boolean isEndYPerc() {
		return endYperc;
	}
	
	public boolean isStartAdjust() {
		return startAdjust;
	}

	public void setStartAdjust(boolean startAdjust) {
		this.startAdjust = startAdjust;
		this.endAdjust = false;
		fireStateChanged();
	}

	public boolean isEndAdjust() {
		return endAdjust;
	}

	public void setEndAdjust(boolean endAdjust) {
		this.endAdjust = endAdjust;
		this.startAdjust = false;
		fireStateChanged();
	}
	
	private boolean isValid() {
		return getFractions().length>=2 && (startX!=endX || startY!=endY);
	}
}
