package ch.lantersoft.grd.protector;
import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import ch.lantersoft.grd.Fraction;
import ch.lantersoft.grd.ui.CycleMode;


public abstract class MultipleGradientProtector extends PaintProtector {

	private List<Fraction> fractions; // always needs to be sorted
	private List<Fraction> selection;
	private Observer fractionObserver;
	
	private CycleMode cycleMode = CycleMode.REFLECT;
	
	public MultipleGradientProtector() {
		fractions = new ArrayList<Fraction>();
		selection = new ArrayList<Fraction>();
		fractionObserver = new Observer() { // delegate changes
			public void update(Observable o, Object arg) {
				fireStateChanged();
			}
		};
		addFraction(new Fraction(0f,Color.RED));
		addFraction(new Fraction(.5f,Color.GREEN));
		addFraction(new Fraction(1f,Color.BLUE));
	}

	public void addFraction(Fraction fraction) {
		fractions.add(fraction);
		Collections.sort(fractions);
		fraction.addObserver(fractionObserver);
		fireStateChanged();
	}
	
	public void removeSelectedFractions() {
		Fraction[] sel = getSelection();
		for (Fraction f:sel) {
			f.setActive(false);
			fractions.remove(f);
		}
		selection.clear();
		fireStateChanged();
	}
	
	public Fraction[] getFractions() {
		Fraction[] ret = fractions.toArray(new Fraction[fractions.size()]);
		Arrays.sort(ret);
		return ret;
	}

	
	
	public Fraction[] getSelection() {
		return selection.toArray(new Fraction[selection.size()]);
	}
	
	public void setSelection(Fraction frac) {
		for (Fraction f:getSelection())
			removeSelection(f);
		addSelection(frac);
	}
	
	public void addSelection(Fraction frac) {
		frac.setSelected(true);
		selection.add(frac);
	}
	
	public void removeSelection(Fraction frac) {
		frac.setSelected(false);
		selection.remove(frac);
	}

	public CycleMode getCycleMode() {
		return cycleMode;
	}

	public void setCycleMode(CycleMode cycleMode) {
		this.cycleMode = cycleMode;
		fireStateChanged();
	}
}
