package ch.lantersoft.grd.protector;
import java.awt.Paint;
import java.util.Observable;
import java.util.Observer;

import ch.lantersoft.grd.Model;


public class Protector extends Observable {

	private Model model;
	
	private PaintProtector[] paints;
	private int selection;

	public Protector() {
		model = new Model();
		selection = 0;
		paints = new PaintProtector[] {
				new LinearGradientProtector(model),
				new RadialGradientProtector(model)
		};
		Observer observer = new Observer() {
			public void update(Observable o, Object arg) {
				changeAndNotify();
			}
		};
		for (PaintProtector pp:paints) {
			pp.addObserver(observer);
		}
	}
	
	private void changeAndNotify() {
		setChanged();
		notifyObservers();
	}
	
	public LinearGradientProtector getLinearGradientProtector() {
		return (LinearGradientProtector) paints[0];
	}
	
	public RadialGradientProtector getRadialGradientProtector() {
		return (RadialGradientProtector) paints[1];
	}
	
	public Paint getPaint() {
		return paints[selection].getPaint();
	}
	
	public String getCode() {
		return paints[selection].getCode();
	}
	
	public String[] getPaintNames() {
		String[] ret = new String[paints.length];
		for (int i=0;i<paints.length;i++)
			ret[i] = paints[i].getName();
		return ret;
	}

	public int getSelection() {
		return selection;
	}
	
	public void setSelection(int selectedIndex) {
		this.selection = selectedIndex;
		changeAndNotify();
	}
	
	public int getWidth() {
		return model.getWidth();
	}

	public int getHeight() {
		return model.getHeight();
	}
	
	public void setSize(int width, int height) {
		model.setSize(width,height);
		changeAndNotify();
	}

	public void move(int x, int y) {
		paints[selection].move(x, y);
	}
	
	public void click(int x, int y) {
		paints[selection].click(x, y);
	}
}
