package ch.lantersoft.grd.protector;

import java.awt.Color;
import java.awt.Paint;
import java.awt.RadialGradientPaint;

import ch.lantersoft.grd.Fraction;
import ch.lantersoft.grd.Model;

public class RadialGradientProtector extends MultipleGradientProtector {

	private Model model;
	
	private float radius = 40;
	private float centerX = 50;
	private float centerY = 50;
	private float focusX = 75;
	private float focusY = 75;

	private boolean radiusPerc = true;
	private boolean centerXperc = true;
	private boolean centerYperc = true;
	private boolean focusXperc = true;
	private boolean focusYperc = true;
	
	private boolean centerAdjust = false; // never both true
	private boolean focusAdjust = false;
	
	public RadialGradientProtector(Model model) {
		this.model = model;
	}
	
	@Override
	public String getName() {
		return "RadialGradient";
	}

	@Override
	public Paint getPaint() {
		if (!isValid()) return null;
		int n = 0;
		Fraction[] fractions = getFractions();
		for (Fraction f:fractions) {
			if (f.isEnabled()) n++;
		}
		if (n<2) return null;
		float[] fracs = new float[n];
		Color[] colors = new Color[n];
		for (int i=0, j=0;i<fractions.length;i++) {
			if (fractions[i].isEnabled()) {
				fracs[j] = fractions[i].getFraction();
				colors[j] = fractions[i].getColor();
				j++;
			}
		}
		for (int i=0;i<n-1;i++) {
			if (fracs[i+1]>.5f) break;
			if (fracs[i]>=fracs[i+1]) {
				fracs[i+1] = fracs[i]+.001f;
			}
		}
		for (int i=n-1;i>0;i--) {
			if (fracs[i]<.5f) break;
			if (fracs[i-1]>=fracs[i]) {
				fracs[i-1] = fracs[i]-.001f;
			}
		}
		if (fracs[fracs.length-1]>1f) return null;
		float width = model.getWidth();
		float height = model.getHeight();
		float r = radiusPerc?width*radius/100:radius;
		float cx = centerXperc?width*centerX/100:centerX;
		float cy = centerYperc?height*centerY/100:centerY;
		float fx = focusXperc?width*focusX/100:focusX;
		float fy = focusYperc?height*focusY/100:focusY;
		if (r<=0) return null;
		RadialGradientPaint paint = new RadialGradientPaint(
				cx, cy, r, 
				fx, fy, 
				fracs, colors, 
				getCycleMode().getCycleMethod());
		return paint;
	}
	
	@Override
	public String getCode() {
		int n = 0;
		Fraction[] fractions = getFractions();
		for (Fraction f:fractions) {
			if (f.isEnabled()) n++;
		}
		StringBuffer fracs = new StringBuffer();
		StringBuffer colors = new StringBuffer();
		
		int iMax = fractions.length - 1;
        for (int i = 0; ; i++) {
        	fracs.append(fractions[i].getFraction()+"f");
        	colors.append("new Color("+fractions[i].getRed()+", "+fractions[i].getGreen()+", "+fractions[i].getBlue()+")");
		    if (i == iMax) break;
		    fracs.append(", ");
		    colors.append(", ");
        }
		float width = model.getWidth();
		float height = model.getHeight();
		float r = radiusPerc?width*radius/100:radius;
		float cx = centerXperc?width*centerX/100:centerX;
		float cy = centerYperc?height*centerY/100:centerY;
		float fx = focusXperc?width*focusX/100:focusX;
		float fy = focusYperc?height*focusY/100:focusY;
		String ret = "new RadialGradientPaint(\n"
					+ "	" + cx+"f, "+cy+"f, "+r+"f, "+fx+"f, "+fy+"f,\n"
					+ "	" + "new float[] {" + fracs.toString() + "},\n"
					+ "	" + "new Color[] {" + colors.toString() + "},\n"
					+ "	" + "CycleMethod."+getCycleMode().getCycleMethod().toString() + ");";
		return ret;
	}

	@Override
	public void click(int x, int y) {
		if (centerAdjust) {
			centerX = centerXperc?x*100f/model.getWidth():x;
			centerY = centerYperc?y*100f/model.getHeight():y;
			centerAdjust = false;
			fireStateChanged();
		} else if (focusAdjust) {
			focusX = focusXperc?x*100f/model.getWidth():x;
			focusY = focusYperc?y*100f/model.getHeight():y;
			focusAdjust = false;
			fireStateChanged();
		}
	}

	@Override
	public void move(int x, int y) {
		if (centerAdjust) {
			centerX = centerXperc?x*100f/model.getWidth():x;
			centerY = centerYperc?y*100f/model.getHeight():y;
			fireStateChanged();
		} else if (focusAdjust) {
			focusX = focusXperc?x*100f/model.getWidth():x;
			focusY = focusYperc?y*100f/model.getHeight():y;
			fireStateChanged();
		}
	}
	
	public float getRadius() {
		return radius;
	}

	public void setRadius(float radius, boolean perc) {
		this.radius = radius;
		this.radiusPerc = perc;
		fireStateChanged();
	}
	
	public boolean isRadiusPerc() {
		return radiusPerc;
	}
	
	public float getCenterX() {
		return centerX;
	}

	public void setCenterX(float centerX, boolean perc) {
		this.centerX = centerX;
		this.centerXperc = perc;
		fireStateChanged();
	}
	
	public boolean isCenterXPerc() {
		return centerXperc;
	}

	public float getCenterY() {
		return centerY;
	}

	public void setCenterY(float centerY, boolean perc) {
		this.centerY = centerY;
		this.centerYperc = perc;
		fireStateChanged();
	}
	
	public boolean isCenterYPerc() {
		return centerYperc;
	}

	public float getFocusX() {
		return focusX;
	}

	public void setFocusX(float focusX, boolean perc) {
		this.focusX = focusX;
		this.focusXperc = perc;
		fireStateChanged();
	}
	
	public boolean isFocusXPerc() {
		return focusXperc;
	}

	public float getFocusY() {
		return focusY;
	}

	public void setFocusY(float focusY, boolean perc) {
		this.focusY = focusY;
		this.focusYperc = perc;
		fireStateChanged();
	}
	
	public boolean isFocusYPerc() {
		return focusYperc;
	}
	
	public boolean isCenterAdjust() {
		return centerAdjust;
	}

	public void setCenterAdjust(boolean centerAdjust) {
		this.centerAdjust = centerAdjust;
		this.focusAdjust = false;
		fireStateChanged();
	}

	public boolean isFocusAdjust() {
		return focusAdjust;
	}

	public void setFocusAdjust(boolean focusAdjust) {
		this.focusAdjust = focusAdjust;
		this.centerAdjust = false;
		fireStateChanged();
	}
	
	
	private boolean isValid() {
		return getFractions().length>=2;
	}

}
