package ch.lantersoft.grd.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ch.lantersoft.grd.Fraction;
import ch.lantersoft.grd.ui.util.GridBagLayoutComponent;
import ch.lantersoft.grd.ui.util.SmartSpinnerNumberModel;

class CmpGradientFraction extends JToggleButton {
	
	private static final long serialVersionUID = 201008311736L;

	private Fraction fraction;
	
	private JCheckBox cbxActive;
	private JSpinner spnFrac;
	private JSpinner spnRed;
	private JSpinner spnGreen;
	private JSpinner spnBlue;
	private JSlider sldFrac;
	private JSlider sldRed;
	private JSlider sldGreen;
	private JSlider sldBlue;
	private JPanel pnlColor;
	
	public CmpGradientFraction(final Fraction fraction) {
		this.fraction = fraction;
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(0,5,0,5));
		setAlignmentX(JComponent.LEFT_ALIGNMENT);
		
		pnlColor = new JPanel(null);
		pnlColor.setBackground(fraction.getColor());
		
		cbxActive = new JCheckBox("",fraction.isEnabled());
		cbxActive.setOpaque(false);
		
		spnFrac = new JSpinner(new SmartSpinnerNumberModel(fraction.getFraction(), 0d, 1d, .01d));
		sldFrac = new JSlider(0, Fraction.MAX_FRAC, (int) (fraction.getFraction()*Fraction.MAX_FRAC));
		sldFrac.setOpaque(false);
		
		spnRed = new JSpinner(new SpinnerNumberModel(fraction.getRed(), 0, 255, 1));
		spnGreen = new JSpinner(new SpinnerNumberModel(fraction.getGreen(), 0, 255, 1));
		spnBlue= new JSpinner(new SpinnerNumberModel(fraction.getBlue(), 0, 255, 1));
		
		sldRed = new JSlider(0,255,fraction.getRed());
		sldGreen = new JSlider(0,255,fraction.getGreen());
		sldBlue = new JSlider(0,255,fraction.getBlue());
		
		sldRed.setOpaque(false);
		sldGreen.setOpaque(false);
		sldBlue.setOpaque(false);
		
		GridBagLayoutComponent cmpColor = new GridBagLayoutComponent();

		cmpColor.addComponent(cbxActive,1,0,1,1,0,0);
		cmpColor.addComponent(sldFrac,2,0,1,1,1,0);
		cmpColor.addComponent(spnFrac,3,0,1,1,0,0);
		cmpColor.addComponent(pnlColor,1,0,3,1,0,0);
		cmpColor.addComponent(new JLabel("Red: "),1,1,1,1,0,0);
		cmpColor.addComponent(sldRed,2,1,1,1,1,0);
		cmpColor.addComponent(spnRed,3,1,1,1,0,0);
		cmpColor.addComponent(new JLabel("Green: "),1,2,1,1,0,0);
		cmpColor.addComponent(sldGreen,2,2,1,1,0,0);
		cmpColor.addComponent(spnGreen,3,2,1,1,1,0);
		cmpColor.addComponent(new JLabel("Blue: "),1,3,1,1,0,0);
		cmpColor.addComponent(sldBlue,2,3,1,1,1,0);
		cmpColor.addComponent(spnBlue,3,3,1,1,0,0);

		// don't let mouseevents reach the togglebutton below
		Component southFiller = Box.createVerticalStrut(5);
		southFiller.addMouseListener(new MouseAdapter() { });
		Component northFiller = Box.createVerticalStrut(5);
		northFiller.addMouseListener(new MouseAdapter() { });
		
		add(northFiller,BorderLayout.NORTH);
		add(cmpColor,BorderLayout.CENTER);
		add(southFiller,BorderLayout.SOUTH);
		
		fraction.addObserver(new Observer() {
			public void update(Observable o, Object arg) {
				CmpGradientFraction.this.update();
			}
		});
		
		sldFrac.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setFraction(sldFrac.getValue()/(float) Fraction.MAX_FRAC);
			}});
		
		sldRed.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setRed(sldRed.getValue());
			}});
		
		sldGreen.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setGreen(sldGreen.getValue());
			}});
		
		sldBlue.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setBlue(sldBlue.getValue());
			}});
		
		spnRed.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setRed((Integer) spnRed.getValue());
			}});
		
		spnGreen.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setGreen((Integer) spnGreen.getValue());
			}});
		
		spnBlue.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setBlue((Integer) spnBlue.getValue());
			}});
		
		spnFrac.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fraction.setFraction(((Double) spnFrac.getValue()).floatValue());
			}});
		
		cbxActive.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fraction.setEnabled(cbxActive.isSelected());
			}});
	}
	
	private void update() {
		spnRed.setValue(fraction.getRed());
		spnGreen.setValue(fraction.getGreen());
		spnBlue.setValue(fraction.getBlue());
		
		sldRed.setValue(fraction.getRed());
		sldGreen.setValue(fraction.getGreen());
		sldBlue.setValue(fraction.getBlue());

		sldFrac.setValue((int) (fraction.getFraction()*Fraction.MAX_FRAC));
		spnFrac.setValue((double) fraction.getFraction());
		
		pnlColor.setBackground(fraction.getColor());

		setSelected(fraction.isSelected());
		
		if (!fraction.isActive()) { // delete yourself
			if (getParent()!=null)
				getParent().remove(this);
		}
		
		repaint();
	}
	
	public Fraction getFraction() {
		return fraction;
	}

	@Override
	public Dimension getMaximumSize() {
		return getPreferredSize();
	}
}