package ch.lantersoft.grd.ui;


import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ch.lantersoft.grd.protector.LinearGradientProtector;
import ch.lantersoft.grd.ui.util.GridBagLayoutComponent;
import ch.lantersoft.grd.ui.util.PercentSpinner;


public class CtrlLinearGradient extends JPanel {

	private static final long serialVersionUID = 201008301835L;
	
	public CtrlLinearGradient(final LinearGradientProtector lgp) {
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(5,5,5,5));
		
		GridBagLayoutComponent cmpPosition = new GridBagLayoutComponent();
		final PercentSpinner spnStartX = new PercentSpinner((int) Math.round(lgp.getStartX()),lgp.isStartXPerc());
		final PercentSpinner spnStartY = new PercentSpinner((int) Math.round(lgp.getStartY()),lgp.isStartYPerc());
		final PercentSpinner spnEndX = new PercentSpinner((int) Math.round(lgp.getEndX()),lgp.isEndXPerc());
		final PercentSpinner spnEndY = new PercentSpinner((int) Math.round(lgp.getEndY()),lgp.isEndYPerc());

		final JToggleButton tbnStartAdjust = new JToggleButton("Adjust");
		final JToggleButton tbnEndAdjust = new JToggleButton("Adjust");
		
		final JComboBox cboCycleMethod = new JComboBox(CycleMode.values());
		cboCycleMethod.setSelectedItem(lgp.getCycleMode());
		
		cmpPosition.addComponent(new JLabel(" Start X"),1,1,1,1,0,0);
		cmpPosition.addComponent(new JLabel(" Start Y"),2,1,1,1,0,0);
		cmpPosition.addComponent(spnStartX,1,2,1,1,.1,0);
		cmpPosition.addComponent(spnStartY,2,2,1,1,.1,0);
		cmpPosition.addComponent(tbnStartAdjust,4,2,1,1,0,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,3,1,1,0,0);

		cmpPosition.addComponent(new JLabel(" End X"),1,4,1,1,0,0);
		cmpPosition.addComponent(new JLabel(" End Y"),2,4,1,1,0,0);
		cmpPosition.addComponent(spnEndX,1,5,1,1,.1,0);
		cmpPosition.addComponent(spnEndY,2,5,1,1,.1,0);
		cmpPosition.addComponent(tbnEndAdjust,4,5,1,1,0,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,6,1,1,0,0);

		cmpPosition.addComponent(cboCycleMethod,1,7,4,1,0,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,8,1,1,0,0);
		
		cmpPosition.addComponent(Box.createHorizontalStrut(5),3,1,1,6,0,0);
		cmpPosition.addComponent(Box.createHorizontalGlue(),5,1,1,6,1,0);
		
		
		add(cmpPosition,BorderLayout.NORTH);
		add(new CmpFractionContainer(lgp),BorderLayout.CENTER);
		
		spnStartX.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				lgp.setStartX(spnStartX.getIntValue(), spnStartX.isPercent());
			} });
		
		spnStartY.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				lgp.setStartY(spnStartY.getIntValue(), spnStartY.isPercent());
			} });
		
		spnEndX.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				lgp.setEndX(spnEndX.getIntValue(), spnEndX.isPercent());
			} });
		
		spnEndY.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				lgp.setEndY(spnEndY.getIntValue(), spnEndY.isPercent());
			} });
		
		tbnStartAdjust.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lgp.setStartAdjust(tbnStartAdjust.isSelected());
				tbnStartAdjust.setSelected(lgp.isStartAdjust());
				tbnEndAdjust.setSelected(lgp.isEndAdjust());
			}
		});
		
		tbnEndAdjust.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lgp.setEndAdjust(tbnEndAdjust.isSelected());
				tbnStartAdjust.setSelected(lgp.isStartAdjust());
				tbnEndAdjust.setSelected(lgp.isEndAdjust());
			}
		});
		
		cboCycleMethod.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange()==ItemEvent.SELECTED) {
					lgp.setCycleMode((CycleMode) cboCycleMethod.getSelectedItem());
				}
			}
		});
		
		lgp.addObserver(new Observer() {
			public void update(Observable o, Object arg) {
				tbnStartAdjust.setSelected(lgp.isStartAdjust());
				tbnEndAdjust.setSelected(lgp.isEndAdjust());
				spnStartX.setIntValue((int) Math.round(lgp.getStartX()));
				spnStartY.setIntValue((int) Math.round(lgp.getStartY()));
				spnEndX.setIntValue((int) Math.round(lgp.getEndX()));
				spnEndY.setIntValue((int) Math.round(lgp.getEndY()));
			}
		});
	}
}
