package ch.lantersoft.grd.ui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JToggleButton;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ch.lantersoft.grd.protector.RadialGradientProtector;
import ch.lantersoft.grd.ui.util.GridBagLayoutComponent;
import ch.lantersoft.grd.ui.util.PercentSpinner;

public class CtrlRadialGradient extends JComponent {

	private static final long serialVersionUID = 201008311126L;
	
	public CtrlRadialGradient(final RadialGradientProtector rgp) {
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(5,5,5,5));
		
		GridBagLayoutComponent cmpPosition = new GridBagLayoutComponent();
		final PercentSpinner spnCenterX = new PercentSpinner((int) Math.round(rgp.getCenterX()),rgp.isCenterXPerc());
		final PercentSpinner spnCenterY = new PercentSpinner((int) Math.round(rgp.getCenterY()),rgp.isCenterYPerc());
		final PercentSpinner spnFocusX = new PercentSpinner((int) Math.round(rgp.getFocusX()),rgp.isFocusXPerc());
		final PercentSpinner spnFocusY = new PercentSpinner((int) Math.round(rgp.getFocusY()),rgp.isFocusYPerc());

		final PercentSpinner spnRadius = new PercentSpinner((int) Math.round(rgp.getRadius()),rgp.isRadiusPerc());

		final JToggleButton tbnCenterAdjust = new JToggleButton("Adjust");
		final JToggleButton tbnFocusAdjust = new JToggleButton("Adjust");
		
		final JComboBox cboCycleMethod = new JComboBox(CycleMode.values());
		cboCycleMethod.setSelectedItem(rgp.getCycleMode());
		
		cmpPosition.addComponent(new JLabel(" Center X"),1,1,1,1,0,0);
		cmpPosition.addComponent(new JLabel(" Center Y"),2,1,1,1,0,0);
		cmpPosition.addComponent(spnCenterX,1,2,1,1,.1,0);
		cmpPosition.addComponent(spnCenterY,2,2,1,1,.1,0);
		cmpPosition.addComponent(tbnCenterAdjust,4,2,1,1,0,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,3,1,1,0,0);

		cmpPosition.addComponent(new JLabel(" Focus X"),1,4,1,1,0,0);
		cmpPosition.addComponent(new JLabel(" Focus Y"),2,4,1,1,0,0);
		cmpPosition.addComponent(spnFocusX,1,5,1,1,.1,0);
		cmpPosition.addComponent(spnFocusY,2,5,1,1,.1,0);
		cmpPosition.addComponent(tbnFocusAdjust,4,5,1,1,0,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,6,1,1,0,0);

		cmpPosition.addComponent(cboCycleMethod,1,7,4,1,0,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,8,1,1,0,0);

		cmpPosition.addComponent(new JLabel(" Radius"),1,9,1,1,0,0);
		cmpPosition.addComponent(spnRadius,1,10,1,1,.1,0);
		cmpPosition.addComponent(Box.createVerticalStrut(10),1,11,1,1,0,0);
		
		cmpPosition.addComponent(Box.createHorizontalStrut(5),3,1,1,6,0,0);
		cmpPosition.addComponent(Box.createHorizontalGlue(),5,1,1,6,1,0);
		
		
		add(cmpPosition,BorderLayout.NORTH);
		add(new CmpFractionContainer(rgp),BorderLayout.CENTER);

		spnRadius.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rgp.setRadius(spnRadius.getIntValue(), spnRadius.isPercent());
			} });
		
		spnCenterX.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rgp.setCenterX(spnCenterX.getIntValue(), spnCenterX.isPercent());
			} });
		
		spnCenterY.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rgp.setCenterY(spnCenterY.getIntValue(), spnCenterY.isPercent());
			} });
		
		spnFocusX.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rgp.setFocusX(spnFocusX.getIntValue(), spnFocusX.isPercent());
			} });
		
		spnFocusY.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rgp.setFocusY(spnFocusY.getIntValue(), spnFocusY.isPercent());
			} });
		
		tbnCenterAdjust.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rgp.setCenterAdjust(tbnCenterAdjust.isSelected());
				tbnCenterAdjust.setSelected(rgp.isCenterAdjust());
				tbnFocusAdjust.setSelected(rgp.isFocusAdjust());
			}
		});
		
		tbnFocusAdjust.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rgp.setFocusAdjust(tbnFocusAdjust.isSelected());
				tbnCenterAdjust.setSelected(rgp.isCenterAdjust());
				tbnFocusAdjust.setSelected(rgp.isFocusAdjust());
			}
		});
		
		cboCycleMethod.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange()==ItemEvent.SELECTED) {
					rgp.setCycleMode((CycleMode) cboCycleMethod.getSelectedItem());
				}
			}
		});
		
		rgp.addObserver(new Observer() {
			public void update(Observable o, Object arg) {
				tbnCenterAdjust.setSelected(rgp.isCenterAdjust());
				tbnFocusAdjust.setSelected(rgp.isFocusAdjust());
				spnCenterX.setIntValue((int) Math.round(rgp.getCenterX()));
				spnCenterY.setIntValue((int) Math.round(rgp.getCenterY()));
				spnFocusX.setIntValue((int) Math.round(rgp.getFocusX()));
				spnFocusY.setIntValue((int) Math.round(rgp.getFocusY()));
			}
		});
	}

}
