package ch.lantersoft.grd.ui.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;

public class FloatTextField extends JTextField implements FocusListener, ActionListener {

	private static final long serialVersionUID = 201009011435L;
	
	private String lastValue; // last valid value
	
	public FloatTextField() {
		this(0f);
	}
	
	public FloatTextField(float f) {
		super(String.valueOf(f),4);
		addFocusListener(this);
		lastValue = getText();
	}
	
	public float getValue() {
		try {
			return Float.parseFloat(getText());
		} catch (NumberFormatException e) {
			return Float.parseFloat(lastValue);
		}
	}
	
	public void setValue(float f) {
		setText(String.valueOf(f));
	}

	@Override
	public void focusGained(FocusEvent e) { }

	@Override
	public void focusLost(FocusEvent e) {
		validate();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		validateText();
	}

	private void validateText() {
		String text = getText();
		try {
			float val = Float.parseFloat(text);
			if (0f <= val && val <=1f) {
				lastValue = getText();
			} else {
				setText(lastValue);
			}
		} catch (NumberFormatException e) { // text is not a float
			setText(lastValue);
		}
	}
	
}
