package ch.lantersoft.grd.ui.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractSpinnerModel;
import javax.swing.JFormattedTextField;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class PercentSpinner extends JSpinner {

	private static final long serialVersionUID = 201008311540L;
	
	private PercentSpinnerModel model;
	private PercentSpinnerEditor editor; 
	
	public PercentSpinner(int value, boolean perc) {
		this.model = new PercentSpinnerModel(value, perc);
		this.editor = new PercentSpinnerEditor(model); 
		setModel(model);
		setEditor(editor);
		addChangeListener(editor);
	}
	
	public int getIntValue() {
		return model.value;
	}
	
	public void setIntValue(int val) {
		model.setValue(val);
	}
	
	public boolean isPercent() {
		return model.percent;
	}
	
	public void setPercent(boolean b) {
		model.setPercent(b);
	}
	
	private static class PercentSpinnerEditor extends JFormattedTextField implements ChangeListener, PropertyChangeListener, ActionListener {

		private static final long serialVersionUID = 201008311617L;
		
		private PercentSpinnerModel model;
		
		public PercentSpinnerEditor(PercentSpinnerModel model) {
			this.model = model;
			setText(model.getText());
			addPropertyChangeListener(this);
			addActionListener(this);
		}
		
		public void actionPerformed(ActionEvent event) {
			update();
		}
		
		public void propertyChange(PropertyChangeEvent evt) {
			update();
		}
		
		private void update() {
			String text = getText();
			if (text.matches("\\-?\\d+%?")) {
				if (text.contains("%")) {
					model.setPercent(true);
				} else {
					model.setPercent(false);
				}
				model.setValue(text);
			} else { // restore text
				setText(model.getText());
			}
		}

		public void stateChanged(ChangeEvent e) {
		    setText(model.getText());
		}
	}
	
	private static class PercentSpinnerModel extends AbstractSpinnerModel {

		private int value; // or Integer or Number
		private boolean percent;
		
		public PercentSpinnerModel(int value, boolean perc) {
			this.value = value;
			this.percent = perc;
		}
		
		public Object getValue() {
			return value;
		}

		/**
		 * Here only the value is set, not whether it is in percent or not.
		 */
		public void setValue(Object value) {
			String text = value.toString();
			if (text.contains("%")) 
				text = text.substring(0,text.indexOf("%"));
			int val = Integer.parseInt(text);
			if (this.value!=val) {
				this.value = val;
				fireStateChanged();
			}
		}

		public Object getNextValue() {
			return value+1;
		}

		public Object getPreviousValue() {
			return value-1;
		}
		
		public boolean isPercent() {
			return percent;
		}
		
		public void setPercent(boolean b) {
			if (this.percent!=b) {
				this.percent = b;
				fireStateChanged();
			}
		}

		public String getText() {
			return String.valueOf(value)+(isPercent()?"%":"");
		}
	}
}
