package ch.lantersoft.grd.ui.util;

import javax.swing.SpinnerNumberModel;

/**
 * If the next value is higher than the maximum, instead of null, the maximum
 * will be taken. Same for the minimum. That actually justifies the name "smart"
 * pretty much compared to the crampy hack we're being provided from swing.
 */
public class SmartSpinnerNumberModel extends SpinnerNumberModel {

	public SmartSpinnerNumberModel(Number value, Comparable<Number> minimum, Comparable<Number> maximum, Number stepSize) {
		super(value,minimum,maximum,stepSize);
	}

	public SmartSpinnerNumberModel(int value, int minimum, int maximum, int stepSize) {
		super(value,minimum, maximum, stepSize);
	}

	public SmartSpinnerNumberModel(double value, double minimum, double maximum, double stepSize) {
		super(value, minimum, maximum, stepSize);
	}

	public SmartSpinnerNumberModel() {
		super();
	}
	 
	@Override
	public Object getNextValue() {
		return incrValue(1);
	}

	@Override
	public Object getPreviousValue() {
		return incrValue(-1);
	}
	
	@SuppressWarnings("unchecked")
	private Number incrValue(int dir) {
		Number value = (Number) getValue();
		Number stepSize = (Number) getStepSize();
		Comparable<Number> maximum = (Comparable<Number>) getMaximum();
		Comparable<Number> minimum = (Comparable<Number>) getMinimum();
		Number newValue;
		if ((value instanceof Float) || (value instanceof Double)) {
			double v = value.doubleValue()
					+ (stepSize.doubleValue() * (double) dir);
			if (value instanceof Double) {
				newValue = new Double(v);
			} else {
				newValue = new Float(v);
			}
		} else {
			long v = value.longValue() + (stepSize.longValue() * (long) dir);

			if (value instanceof Long) {
				newValue = new Long(v);
			} else if (value instanceof Integer) {
				newValue = new Integer((int) v);
			} else if (value instanceof Short) {
				newValue = new Short((short) v);
			} else {
				newValue = new Byte((byte) v);
			}
		}

		if ((maximum != null) && (maximum.compareTo(newValue) < 0)) {
			return (Number) maximum;
		}
		if ((minimum != null) && (minimum.compareTo(newValue) > 0)) {
			return (Number) minimum;
		} else {
			return newValue;
		}
	}
}
